Package: aRtsy
Title: Generative Art with 'ggplot2'
Description: Provides algorithms for creating artworks in the 'ggplot2' language that incorporate some form of randomness.
Version: 1.0.1
Date: 2025-05-16
Authors@R: 
    person(given = "Koen",
           family = "Derks",
           role = c("aut", "cre"),
           email = "koen-derks@hotmail.com")
BugReports: https://github.com/koenderks/aRtsy/issues
URL: https://koenderks.github.io/aRtsy/,
        https://github.com/koenderks/aRtsy,
        https://twitter.com/aRtsy_package,
        https://mastodon.social/@aRtsy_package
Suggests: testthat (>= 3.0.0)
Imports: ambient, e1071, ggplot2 (>= 3.4.0), FNN, randomForest, Rcpp,
        scales, stats
LinkingTo: Rcpp, RcppArmadillo
Language: en-US
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-05-16 06:06:17 UTC; koenderks
Author: Koen Derks [aut, cre]
Maintainer: Koen Derks <koen-derks@hotmail.com>
Repository: CRAN
Date/Publication: 2025-05-16 08:50:28 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:40:45 UTC; unix
Archs: aRtsy.so.dSYM
