Package: aRtsy
Title: Generative Art with 'ggplot2'
Description: Provides algorithms for creating artworks in the 'ggplot2' language that incorporate some form of randomness.
Version: 1.0.0
Date: 2024-11-05
Authors@R: 
    person(given = "Koen",
           family = "Derks",
           role = c("aut", "cre"),
           email = "koen-derks@hotmail.com")
BugReports: https://github.com/koenderks/aRtsy/issues
URL: https://koenderks.github.io/aRtsy/, https://github.com/koenderks/aRtsy, https://twitter.com/aRtsy_package, https://botsin.space/web/@aRtsy_package
Suggests:
    testthat (>= 3.0.0)
Imports:
    ambient,
    e1071,
    ggplot2 (>= 3.4.0),
    kknn,
    randomForest,
    Rcpp,
    scales,
    stats
LinkingTo:
    Rcpp,
    RcppArmadillo
Language: en-US
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Config/testthat/edition: 3
