# Generated by roxygen2: do not edit by hand

export(canvas_ant)
export(canvas_blacklight)
export(canvas_chladni)
export(canvas_circlemap)
export(canvas_cobweb)
export(canvas_collatz)
export(canvas_diamonds)
export(canvas_flame)
export(canvas_flow)
export(canvas_forest)
export(canvas_function)
export(canvas_gemstone)
export(canvas_lissajous)
export(canvas_mandelbrot)
export(canvas_maze)
export(canvas_mesh)
export(canvas_mosaic)
export(canvas_nebula)
export(canvas_petri)
export(canvas_phyllotaxis)
export(canvas_planet)
export(canvas_polylines)
export(canvas_recaman)
export(canvas_ribbons)
export(canvas_segments)
export(canvas_smoke)
export(canvas_splits)
export(canvas_squares)
export(canvas_stripes)
export(canvas_strokes)
export(canvas_swirls)
export(canvas_tiles)
export(canvas_turmite)
export(canvas_watercolors)
export(colorPalette)
export(saveCanvas)
export(theme_canvas)
import(Rcpp)
importFrom(stats,predict)
useDynLib(aRtsy)
