# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_ant <- function(canvas, directions, iterations, resolution) {
    .Call('_aRtsy_cpp_ant', PACKAGE = 'aRtsy', canvas, directions, iterations, resolution)
}

cpp_chladni <- function(x, y, waves) {
    .Call('_aRtsy_cpp_chladni', PACKAGE = 'aRtsy', x, y, waves)
}

cpp_circlemap <- function(canvas, left, right, bottom, top, iters) {
    .Call('_aRtsy_cpp_circlemap', PACKAGE = 'aRtsy', canvas, left, right, bottom, top, iters)
}

cpp_collatz_sequence <- function(x) {
    .Call('_aRtsy_cpp_collatz_sequence', PACKAGE = 'aRtsy', x)
}

cpp_collatz <- function(empty, series, even, odd) {
    .Call('_aRtsy_cpp_collatz', PACKAGE = 'aRtsy', empty, series, even, odd)
}

cpp_flame <- function(canvas, iterations, resolution, edge, blend, weighted, post, final, extra, colors, functions, funcWeights, funcPars, variations, varWeights, varParams, postPars, finalPars, extraPars, bsym) {
    .Call('_aRtsy_cpp_flame', PACKAGE = 'aRtsy', canvas, iterations, resolution, edge, blend, weighted, post, final, extra, colors, functions, funcWeights, funcPars, variations, varWeights, varParams, postPars, finalPars, extraPars, bsym)
}

cpp_flow <- function(canvas, angles, lines, iters, ncolors, left, right, top, bottom, stepmax) {
    .Call('_aRtsy_cpp_flow', PACKAGE = 'aRtsy', canvas, angles, lines, iters, ncolors, left, right, top, bottom, stepmax)
}

cpp_maze <- function(X, x, y) {
    .Call('_aRtsy_cpp_maze', PACKAGE = 'aRtsy', X, x, y)
}

cpp_mesh <- function(canvas, points, centers, iterations, start, order, radii, increase) {
    .Call('_aRtsy_cpp_mesh', PACKAGE = 'aRtsy', canvas, points, centers, iterations, start, order, radii, increase)
}

cpp_petri_closest <- function(attractor_x, attractor_y, nodes_x, nodes_y, attraction_distance) {
    .Call('_aRtsy_cpp_petri_closest', PACKAGE = 'aRtsy', attractor_x, attractor_y, nodes_x, nodes_y, attraction_distance)
}

cpp_petri_kill <- function(attractor_x, attractor_y, nodes_x, nodes_y, kill_distance) {
    .Call('_aRtsy_cpp_petri_kill', PACKAGE = 'aRtsy', attractor_x, attractor_y, nodes_x, nodes_y, kill_distance)
}

cpp_draw_circle <- function(center_x, center_y, diameter, n) {
    .Call('_aRtsy_cpp_draw_circle', PACKAGE = 'aRtsy', center_x, center_y, diameter, n)
}

cpp_petri_directions <- function(attractor_x, attractor_y, nodes_x, nodes_y, closest_nodes) {
    .Call('_aRtsy_cpp_petri_directions', PACKAGE = 'aRtsy', attractor_x, attractor_y, nodes_x, nodes_y, closest_nodes)
}

cpp_phyllotaxis <- function(iter, a, p) {
    .Call('_aRtsy_cpp_phyllotaxis', PACKAGE = 'aRtsy', iter, a, p)
}

cpp_planet <- function(canvas, resolution, radius, xcenter, ycenter, threshold, iterations, ncolors, colorsused, starprob, fade, lightright) {
    .Call('_aRtsy_cpp_planet', PACKAGE = 'aRtsy', canvas, resolution, radius, xcenter, ycenter, threshold, iterations, ncolors, colorsused, starprob, fade, lightright)
}

cpp_polylines <- function(canvas, ratio, iters, rows, cols) {
    .Call('_aRtsy_cpp_polylines', PACKAGE = 'aRtsy', canvas, ratio, iters, rows, cols)
}

cpp_recaman <- function(n, start, increment) {
    .Call('_aRtsy_cpp_recaman', PACKAGE = 'aRtsy', n, start, increment)
}

cpp_smoke <- function(canvas, coords, color_mat, init, algorithm, shape, all_colors) {
    .Call('_aRtsy_cpp_smoke', PACKAGE = 'aRtsy', canvas, coords, color_mat, init, algorithm, shape, all_colors)
}

cpp_splits <- function(x, xend, y, yend, col, sda, ncols, iterations) {
    .Call('_aRtsy_cpp_splits', PACKAGE = 'aRtsy', x, xend, y, yend, col, sda, ncols, iterations)
}

cpp_squares <- function(X, neighbors, s, cuts, ratio) {
    .Call('_aRtsy_cpp_squares', PACKAGE = 'aRtsy', X, neighbors, s, cuts, ratio)
}

cpp_strokes <- function(canvas, neighbors, s, p) {
    .Call('_aRtsy_cpp_strokes', PACKAGE = 'aRtsy', canvas, neighbors, s, p)
}

cpp_swirls <- function(heightMap, iterations, n, resolution, ncols, lwd, freq) {
    .Call('_aRtsy_cpp_swirls', PACKAGE = 'aRtsy', heightMap, iterations, n, resolution, ncols, lwd, freq)
}

cpp_tiles <- function(canvas, conv, diffa, diffb, feedrate, killrate, iterations) {
    .Call('_aRtsy_cpp_tiles', PACKAGE = 'aRtsy', canvas, conv, diffa, diffb, feedrate, killrate, iterations)
}

cpp_turmite <- function(canvas, iters, row, col, p) {
    .Call('_aRtsy_cpp_turmite', PACKAGE = 'aRtsy', canvas, iters, row, col, p)
}

cpp_watercolors <- function(x, y, s, maxdepth, resolution) {
    .Call('_aRtsy_cpp_watercolors', PACKAGE = 'aRtsy', x, y, s, maxdepth, resolution)
}

cpp_knn <- function(x, y, z, newx, newy, k) {
    .Call('_aRtsy_cpp_knn', PACKAGE = 'aRtsy', x, y, z, newx, newy, k)
}

