## aRtsy version 1.0.0
This is a submission for aRtsy version 1.0.0. In this version I have:

* Fixed a bug in the c++ code.
* Improved documentation.

## Test environments
* Windows install, (on github actions) R release
* OS X install (on github actions), R release
* Linux install (on github actions), R release

## R CMD check results
There were no ERRORs or WARNINGs.

On linux, there was 1 NOTE:

* checking installed package size ... NOTE. installed size is 20.5Mb. sub-directories of 1Mb or more: libs 20.0Mb

## Downstream dependencies
There is currently 1 downstream dependency for this package.
